import os
import glob
import subprocess
from kaldi_helpers.script_utilities import find_files_by_extensions

TEST_FILES_BASE_DIR = os.path.normpath(os.path.join(os.getcwd(), os.path.join(".", "testfiles")))
SCRIPT_PATH = os.path.normpath(os.path.join(os.getcwd(), os.path.join("..", "kaldi_helpers", "trs_to_json.py")))

# all_files_in_dir = list(glob.glob(os.path.join(SCRIPT_PATH, "*.py"), recursive=True))

print(TEST_FILES_BASE_DIR)
print(SCRIPT_PATH)

subprocess.call(["python", SCRIPT_PATH, "--indir", TEST_FILES_BASE_DIR])

# print(all_files_in_dir)


"""
Collection of utilities for working with JSON files.

Copyright: University of Queensland, 2019
Contributors:
             Nicholas Lambourne - (University of Queensland, 2018)
"""

import json
from typing import Union
from _io import TextIOWrapper


def load_json_file(file_name: str) -> object:
    """
    Given a filename (parameter) containing JSON, load and
    return the a list of python dictionaries with containing the same information.
    :param file_name: name of file containing JSON to read from.
    :return a Python dictionary with the contents of the JSON file.
    """
    file = open(file_name, "r", encoding="utf-8")
    data: object = json.load(file)
    return data


def write_data_to_json_file(data: object, output: Union[str, TextIOWrapper]) -> None:
    """
    Writes the given Python dictionary (or list) object to a JSON file at the the given
    output_scripts location (which can either be a file - specified as a string, or
    directed to an output_scripts like sys.stdout or sys.stderr).
    :param data: the Python dictionary to be converted to JSON and written.
    :param output: the file to write the dictionary contents to.
    """
    json_data_string = json.dumps(data,
                                  indent=4,
                                  separators=(',', ': '),
                                  sort_keys=False)
    if isinstance(output, str):
        with open(output, "w") as file:
            file.write(json_data_string)
    else:
        print(json_data_string, file=output, flush=True)



#!/usr/bin/python3

"""
Takes a Praat TextGrid file and converts it to an ELAN (.eaf) file format.

Copyright: University of Queensland, 2019
Contributors:
             Nicholas Lambourne - (University of Queensland, 2018)
"""

from argparse import ArgumentParser
from pathlib import Path
from pympi import Praat, Elan


def get_first_wav(wav_scp: str) -> str:
    with open(wav_scp, "r") as scp:
        return scp.readline().strip().split(" ")[1]


def main() -> None:
    parser: ArgumentParser = ArgumentParser(description="Converts Praat TextGrid format to ELAN eaf Format.")
    parser.add_argument("--tg", "--textgrid", type=str, help="The input_scripts TextGrid format file", required=True)
    parser.add_argument("--wav", type=str, help="The relative path to the .wav file associated with the TextGrid",
                        required=True)
    parser.add_argument("-o", "--outfile", type=str, help="The file path for the ELAN file output_scripts",
                        default="./inferred-aligned.eaf")
    arguments = parser.parse_args()

    textgrid_file = arguments.tg
    wav_file = Path(get_first_wav(arguments.wav))
    output_file = Path(arguments.outfile)

    if not output_file.parent:
        Path.mkdir(output_file.parent, parents=True)

    textgrid = Praat.TextGrid(file_path=textgrid_file)

    elan = textgrid.to_eaf()

    elan.add_linked_file(file_path=str(wav_file.absolute()),
                         relpath=str(wav_file),
                         mimetype=Elan.Eaf.MIMES.get("wav", ""),
                         time_origin=0)

    elan.to_file(output_file)


if __name__ == '__main__':
    main()



import os
from _pytest.capture import CaptureFixture
from kaldi_helpers.input_scripts.clean_json import *
from kaldi_helpers.script_utilities import write_data_to_json_file

EXAMPLE_JSON_DATA = [
    {"transcript": "Comment t'appelles tu?"},
    {"transcript": "Je m'appelle François."},
    {"transcript": "Est-ce tu a une livre préférér."},
    {"transcript": "Oui, j'adore L'histoire secrète par Donna Tartt."},
    {"transcript": "Vraiment? Je n'ai jamais lu ça."},
]


def test_get_english_words() -> None:
    english_words = get_english_words()
    assert "test" in english_words
    assert "français" not in english_words


def test_clean_utterance_remove_english() -> None:
    example_utterance = {"transcript": "je veux une petite dejeuner"}
    english_words = get_english_words()
    cleaned_utterance, english_word_count = clean_utterance(example_utterance,
                                                            remove_english=True,
                                                            english_words=english_words,
                                                            punctuation="…’“–”‘°",
                                                            special_cases=['<silence>'])
    assert cleaned_utterance == ['je', 'veux', 'une']  # Apparently dejeuner is in English?
    assert english_word_count == 2


def test_clean_utterance_keep_english() -> None:
    example_utterance = {"transcript": "I say, jeune homme!"}
    english_words = get_english_words()
    cleaned_utterance, english_word_count = clean_utterance(example_utterance,
                                                            remove_english=True,
                                                            english_words=english_words,
                                                            punctuation="…,’“–”‘°!",
                                                            special_cases=['<silence>'])
    assert cleaned_utterance == ['i', 'say', 'jeune', 'homme']
    assert english_word_count == 0


def test_is_valid_utterance_remove_english() -> None:
    cleaned_utterance = ['je', 'veux', 'acheter', 'la', 'nouveau', 'bonbon', 'pour', 'ma', 'mère',
                         'et', 'mon', 'père']
    langid_identifier = LanguageIdentifier.from_modelstring(model,
                                                            norm_probs=True)
    assert is_valid_utterance(clean_words=cleaned_utterance,
                              english_word_count=0,
                              remove_english=True,
                              use_langid=True,
                              langid_identifier=langid_identifier) is True


def test_is_valid_utterance_keep_english() -> None:
    cleaned_utterance = ['i', 'say', 'jeune', 'homme']
    langid_identifier = LanguageIdentifier.from_modelstring(model,
                                                            norm_probs=True)
    assert is_valid_utterance(clean_words=cleaned_utterance,
                              english_word_count=0,
                              remove_english=False,
                              use_langid=False,
                              langid_identifier=langid_identifier) is True


def test_clean_json_data_full() -> None:
    clean_data = clean_json_data(EXAMPLE_JSON_DATA,
                                 remove_english=True,
                                 use_langid=True)
    print(clean_data)
    assert clean_data == [
        {"transcript": "je mappelle françois"},
        {"transcript": "vraiment je nai jamais lu ça"}
    ]


def test_clean_json_data_full_file() -> None:
    file_in_name = 'file_in.json'
    file_out_name = 'file_out.json'
    # os.system('touch file_out.json') # Not Windows compatible
    open(file_out_name, 'w')
    write_data_to_json_file(EXAMPLE_JSON_DATA, file_in_name)
    os.system(f"clean_json.py --infile {file_in_name} --outfile {file_out_name}"
              f" --removeEng --useLangid")
    # assert load_json_file(file_out_name) == [
    #     {"transcript": "je mappelle françois"},
    #     {"transcript": "vraiment je nai jamais lu ça"}
    # ]
    os.remove(file_in_name)
    os.remove(file_out_name)


def test_clean_json_data_full_command_line(capsys: CaptureFixture) -> None:
    pass



#!/usr/bin/python3

"""
Takes a CTM (time aligned) file and produces an equivalent Praat TextGrid file.

Copyright: University of Queensland, 2019
Contributors:
             Nicholas Lambourne - (University of Queensland, 2018)
"""

from argparse import ArgumentParser
from csv import reader
from pathlib import Path
from typing import Dict, Tuple
from praatio import tgio
import codecs


def ctm_to_dictionary(ctm_file_path: str,
                      segments_dictionary: Dict[str, str]) -> dict:
    with codecs.open(ctm_file_path, encoding="utf8") as file:
        ctm_entries = list(reader(file, delimiter=" "))
    textgrid_dictionary = dict()
    for entry in ctm_entries:
        utterance_id, segment_start_time = segments_dictionary[entry[0]]
        if utterance_id not in textgrid_dictionary:
            textgrid_dictionary[utterance_id] = []
        relative_start_time = float(entry[2])
        absolute_start_time = segment_start_time + relative_start_time
        absolute_end_time = absolute_start_time + float(entry[3])
        inferred_text = entry[4]
        utterance_segment = (str(absolute_start_time),
                             str(absolute_end_time),
                             inferred_text)
        textgrid_dictionary[utterance_id].append(utterance_segment)
    return textgrid_dictionary


def get_segment_dictionary(segment_file_name: str) -> Dict[str, Tuple[str, float]]:
    with open(segment_file_name, "r") as file:
        segment_entries = list(reader(file, delimiter=" "))
    segment_dictionary = dict()
    for entry in segment_entries:
        segment_id = entry[0]
        utterance_id = entry[1]
        start_time = float(entry[2])
        segment_dictionary[segment_id] = (utterance_id, start_time)
    return segment_dictionary


def wav_scp_to_dictionary(scp_file_name: str) -> dict:
    wav_dictionary = dict()
    with open(scp_file_name) as file:
        wav_entries = list(reader(file, delimiter=" "))
        for entry in wav_entries:
            utterance_id = entry[0]
            wav_file_path = entry[1]
            wav_dictionary[utterance_id] = wav_file_path
    return wav_dictionary


def create_textgrid(wav_dictionary: Dict[str, str],
                    ctm_dictionary: dict,
                    output_directory: str) -> None:
    for index, utterance_id in enumerate(wav_dictionary.keys()):
        textgrid = tgio.Textgrid()
        tier = tgio.IntervalTier(name='phones',
                                 entryList=ctm_dictionary[utterance_id],
                                 minT=0,
                                 pairedWav=str(Path(wav_dictionary[utterance_id])))
        textgrid.addTier(tier)
        textgrid.save(str(Path(output_directory, f"utterance-{index}.TextGrid")))


def main() -> None:
    parser: ArgumentParser = ArgumentParser(description="Converts Kaldi CTM format to Praat Textgrid Format.")
    parser.add_argument("-c", "--ctm",
                        type=str,
                        help="The input_scripts CTM format file",
                        required=True)
    parser.add_argument("-w", "--wav",
                        type=str,
                        help="The input_scripts wav.scp file",
                        required=True)
    parser.add_argument("-s", "--seg",
                        type=str,
                        help="The segment to utterance mapping",
                        default="./segments")
    parser.add_argument("-o", "--outdir",
                        type=str,
                        help="The directory path for the Praat TextGrid output_scripts",
                        default=".")
    arguments = parser.parse_args()

    segments_dictionary = get_segment_dictionary(arguments.seg)
    ctm_dictionary = ctm_to_dictionary(arguments.ctm, segments_dictionary)
    wav_dictionary = wav_scp_to_dictionary(arguments.wav)
    output_directory = Path(arguments.outdir)

    if not output_directory.parent:
        Path.mkdir(output_directory.parent, parents=True)

    output_directory = str(output_directory)

    create_textgrid(wav_dictionary,
                    ctm_dictionary,
                    output_directory)


if __name__ == '__main__':
    main()



"""
Collection of utilities for finding and working with files.

Copyright: University of Queensland, 2019
Contributors:
             Nicholas Lambourne - (University of Queensland, 2018)
"""

import glob
import os
from typing import Set, List


def find_files_by_extensions(set_of_all_files: Set[str], extensions: Set[str]) -> Set[str]:
    """
    Searches for all files in the set of files with the given extensions.
    :param set_of_all_files: set of file names in string format
    :param extensions: file extension being searched for
    :return: set of file_names matched with given extension. if none exists, returns an empty set.
    """
    results = set()
    print(set_of_all_files)
    for file_path in set_of_all_files:
        name, extension = os.path.splitext(file_path)
        print(extension)
        if extension in extensions:
            results.add(file_path)
    results.add('.')
    return results


def find_first_file_by_extension(set_of_all_files: List[str], extensions: List[str]) -> str:
    """
    Searches for the first file with a given extension in a set of files.
    :param set_of_all_files: set of file names in string format
    :param extensions: file extension being searched for
    :return: name of the first file_name that is matched, if any. otherwise, this method returns an empty string
    """
    for file_ in set_of_all_files:
        name, extension = os.path.splitext(file_)
        if ("*" + extension.lower()) in extensions:
            return file_
    return ""


def find_all_files_in_dir_by_extensions(directory_path: str, extensions: Set[str]) -> Set[str]:
    """
    Find all files with given extensions in a directory recursively
    :param directory_path: path to directory to recursively search
    :param extensions: the set of allowed extensions
    :return: a set of file paths
    """
    path = str(os.path.join(directory_path, "**"))
    all_files_in_dir: Set[str] = set(glob.glob(path, recursive=True))
    return find_files_by_extensions(all_files_in_dir, extensions)



#!/usr/bin/python3

"""
Given a json file with transcript information this tools can perform
manipulations including generating word lists.

Copyright: University of Queensland, 2019
Contributors:
              Josh Arnold - (The University of Queensland, 2017)
              Ben Foley - (The University of Queensland, 2018)
              Nicholas Lambourne - (The University of Queensland, 2019)
"""

import argparse
import os
import sys
from typing import List, Dict
from kaldi_helpers.script_utilities import load_json_file


def save_word_list(word_list: List[str], file_name: str) -> None:
    """
    Given a list of strings, write them to a new file named filename.
    :param word_list: list of words to write.
    :param file_name: name of file to write word list to.
    """
    with open(file_name, "w", encoding='utf-8') as f:
        for word in word_list:
            f.write(word + "\n",)
        print(f"Wrote word list to {file_name}")


def extract_word_list(json_data: List[Dict[str, str]]) -> List[str]:
    """
    Unpack a dictionary constructed from a json_file - containing the key
    "transcript" - into a (Python) list of words.
    :param json_data: Python list of dictionaries read from a JSON file.
    :return: list of unique words from data, sorted alphabetically.
    """
    result: List[str] = []
    for utterance in json_data:
        words = utterance.get("transcript").split()
        result.extend(words)
    result = list(set(result))
    return sorted(result)


def extract_additional_words(file_name: str) -> List[str]:
    """
    Extracts words from an additional text file for the purpose
    of extending the lexicon to words that there is no sound data for.
    :param file_name: the name of the file to extract words from.
    :return: a list of words
    """
    words = []
    if os.path.exists(file_name):
        with open(file_name, "r") as f:
            print(f"Extracting additional words from {file_name}")
            for line in f.readlines():
                new_words = line.strip().split(" ")
                words += [word for word in new_words]
    else:
        print(f"WARNING: Additional word list file at {file_name} does not exist, skipping!")
    return words


def generate_word_list(transcription_file: str,
                       word_list_file: str,
                       output_file: str,
                       kaldi_corpus_file: str) -> None:
    """
    Generates the wordlist.txt file used to populate the Kaldi file structure and generate
    the lexicon.txt file.
    :param transcription_file: path to the json file containing the transcriptions
    :param word_list_file: the path of the file to write the word list to
    :param output_file: the path of the file to write the word list to
    :param kaldi_corpus_file: file path to the corpus.txt created by json_to_kaldi.py
    :return:
    """
    json_data: List[Dict[str, str]] = load_json_file(transcription_file)

    print("Extracting word list(s)...", flush=True, file=sys.stderr)

    # Retrieve ELAN word data
    word_list = extract_word_list(json_data)

    # Add additional words to lexicon if required
    if word_list_file:
        additional_words = extract_additional_words(word_list_file)
        word_list.extend(additional_words)

    if kaldi_corpus_file:
        corpus_file_words = extract_additional_words(kaldi_corpus_file)
        word_list.extend(corpus_file_words)

    # Remove duplicates
    word_list = list(set(word_list))

    print(sorted(word_list))

    print(f"Writing wordlist to file...", flush=True, file=sys.stderr)
    save_word_list(word_list, output_file)


def main():
    """
    Run the entire make_wordlist.py as a command line utility.
    
    Usage: python3 make_wordlist.py [-h] -i INFILE [-o OUTFILE] [-w WORDLIST] [-t TEXTCORPUS] [-c KALDICORPUS]
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--infile",
                        type=str,
                        required=True,
                        help="The json file containing the transcriptions.")
    parser.add_argument("-o", "--outfile",
                        type=str,
                        required=True,
                        help="The path of the file to write the word list to.")
    parser.add_argument("-w", "--word_list",
                        type=str,
                        required=False,
                        help="File path to an optional additional word list.")
    parser.add_argument("-c", "--kaldi_corpus",
                        type=str,
                        help="File path to the corpus.txt created by json_to_kaldi.py.",
                        required=True)
    arguments = parser.parse_args()

    generate_word_list(transcription_file=arguments.infile,
                       word_list_file=arguments.word_list,
                       output_file=arguments.outfile,
                       kaldi_corpus_file=arguments.kaldi_corpus)

    print("Done.", file=sys.stderr)


if __name__ == '__main__':
    main()



#!/usr/bin/python3

"""
Parse a Transcriber (*.trs) file and extract information from it and export in json format.

Copyright: University of Queensland, 2019
Contributors:
              Aninda Saha (University of Queensland, 2018)
              Nicholas Lambourne (University of Queensland, 2019)
"""


import os
import sys
import argparse
import platform
import uuid
import glob
import xml.etree.ElementTree as ET
from typing import Dict, List, Set, Tuple, Union
from kaldi_helpers.script_utilities import find_files_by_extensions, write_data_to_json_file


def conditional_log(condition: bool, text: str) -> None:
    """
    Work around for UTF8 file name and the windows console.

    :param condition: condition to indicate whether text should be output_scripts to stderr
    :param text: text to output_scripts to stderr
    """

    if condition:
        if platform.system() == "Windows":
            sys.stderr.write(text.encode("cp850", errors="backslashreplace").decode(sys.stdout.encoding))
        else:
            sys.stderr.write(text)
        sys.stderr.flush()


def process_trs(file_name: str, verbose_output: bool) -> List[Dict[str, Union[str, float]]]:

    """
    Method to process the trs files and return a list of utterances.

    :param file_name: file_name of the .trs file
    :param verbose_output: whether or not output_scripts to stderr
    :return: a list of dictionaries. Each dictionary contains key information on utterances in the following format:
                                   {'speaker_id': <speaker_id>,
                                    'audio_file_name': <file_name>,
                                    'transcript': <transcription_label>,
                                    'start_ms': <start_time_in_milliseconds>,
                                    'stop_ms': <stop_time_in_milliseconds>}
    """

    conditional_log(verbose_output, "Processing transcript '%s'\n" % file_name)

    utterances: List[Dict[str, Union[str, float]]] = []
    try:
        tree: ET.ElementTree = ET.parse(file_name)
        root: ET.Element = tree.getroot()
        wave_name: str = root.attrib["audio_filename"] + ".wav"
        turn_nodes: List[ET.Element] = tree.findall(".//Turn")
        for turn_node in turn_nodes:
            utterances = utterances + process_turn(wave_name, turn_node, tree)
    except ET.ParseError as error:
        conditional_log(True, "XML parser failed to parse '%s'!\n" % file_name)
        conditional_log(True, str(error))
    return utterances


def process_turn(wave_name: str, turn_node: ET.Element, tree: ET.ElementTree) -> List[Dict[str, Union[str, float]]]:
    """
    Helper method to process each turn_node in the .trs file.

    :param file_name: name of the file
    :param turn_node: the ElementTree node to be processed
    :param wave_name: name of .wav audio file to be processed
    :param tree: XML data represented as a tree data structure
    :return: list of key information on utterances
    """

    turn_end: float = float(turn_node.attrib["endTime"])
    speaker_id: str = turn_node.get("speaker", "")

    speaker_name_node: ET.Element = tree.find(".//Speaker[@id='%s']" % speaker_id)
    if speaker_name_node is not None:
        speaker_name: str = speaker_name_node.attrib["name"]
    else:
        speaker_name: str = str(uuid.uuid4())

    items: List[Tuple[str, str]] = [(element.attrib["time"], element.tail.strip()) for element in turn_node.findall("./Sync")]
    wave_file_name = os.path.join(".", wave_name)

    result: List[Dict[str, Union[str, float]]] = []

    for i in range(0, len(items)):
        time_str, transcription_str = items[i]
        start_time: float = float(time_str)
        if i < len(items) - 1:
            end_time: float = float(items[i + 1][0])
        else:
            end_time = turn_end
        result.append({"speaker_id": speaker_name,
                       "audio_file_name": wave_file_name,
                       "transcript": transcription_str,
                       "start_ms": start_time * 1000.0,
                       "stop_ms": end_time * 1000.0})

    return result


def main() -> None:
    """
    Run the entire trs_to_json.py as a command line utility. It processes the utterances
    and outputs to a file in the same directory as the input_scripts .trs file. The output
    files is named after the basename of the input directory appended with a .json extension.

    Usage: python3 trs_to_json.py [-h] [-d INPUT_DIRECTORY] [-v] > {OUTPUT_FILE]
    """

    parser = argparse.ArgumentParser(description="A command line utility to convert .trs files to .json",
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-i", "--input_dir",
                        type=str,
                        help="Input directory, default='working_dir/input/data'",
                        default="working_dir/input/data/")
    parser.add_argument('-v', '--verbose',
                        type=str,
                        help='More logging to console.',
                        action="store_true")
    parser.add_argument("-o", "--output_json",
                        type=str,
                        help="File name to output_scripts json",
                        default="working_dir/input/output/tmp/")

    arguments: argparse.Namespace = parser.parse_args()

    if arguments.verbose:
        sys.stderr.write(arguments.input_directory + "\n")

    all_files_in_dir: Set[str] = set(glob.glob(os.path.join(arguments.input_dir, "**"), recursive=True))
    transcript_names: Set[str] = find_files_by_extensions(all_files_in_dir, {"*.trs"})

    utterances = []
    for file_name in transcript_names:
        utterances = utterances + process_trs(file_name, arguments.verbose)

    write_data_to_json_file(utterances, arguments.output_json)


if __name__ == '__main__':
    main()



"""
Test script for validating textgrid to json conversion pipeline. 

@author Aninda Saha
"""

from kaldi_helpers.textgrid_to_json import *

SCRIPT_PATH = os.path.join(".", "kaldi_helpers", "textgrid_to_json.py")


def test_process_textgrid_file() -> None:

    speech_tier_total_length = 0
    for (root, dirs, files) in os.walk(TEST_FILES_BASE_DIR):
        for filename in files:
            if filename.endswith(".TextGrid"):
                tg = tgio.openTextgrid(os.path.join(root, filename))
                speech_tier_total_length += len(tg.tierDict['Speech'].entryList)

    intervals: List[Dict[str, Union[str, int]]] = process_textgrid(TEST_FILES_BASE_DIR)
    assert speech_tier_total_length == len(intervals)


def test_textgrid_to_json() -> None:

    utterances: List[Dict[str, Union[str, float]]] = process_textgrid(TEST_FILES_BASE_DIR)
    output_path: str = os.path.join(".", "test", "testfiles")
    result: subprocess.CompletedProcess = subprocess.run(["python", SCRIPT_PATH,
                                                          "--input_dir", TEST_FILES_BASE_DIR,
                                                          "--output_dir", output_path],
                                                         check=True)
    assert result.returncode == 0

    parent_directory_name, base_directory_name = os.path.split(TEST_FILES_BASE_DIR)
    json_name: str = os.path.join(parent_directory_name, base_directory_name + ".json")

    with open(json_name) as f:
        contents: List[Dict[str, Union[str, float]]] = json.loads(f.read())
    assert len(contents) == len(utterances)
    assert utterances == contents

    os.remove(json_name)



#!/usr/bin/python3

"""
Splits a directory of audio (.wav) files into short segments based on silence

Copyright: University of Queensland, 2019
Contributors:
              Nicholas Lambourne - (The University of Queensland, 2019)
"""

from argparse import ArgumentParser
from pathlib import Path
from pydub import AudioSegment
from pydub.silence import split_on_silence
from kaldi_helpers.script_utilities import find_all_files_in_dir_by_extensions


def match_target_amplitude(segment: AudioSegment, target_dbfs) -> AudioSegment:
    """
    Matches an AudioSegment to a specified dBFS level.
    :param segment: AudioSegment to modify
    :param target_dbfs: integer repre
    :return: the segment matched to the target_dbfs
    """
    dbfs_delta = target_dbfs - segment.dBFS
    return segment.apply_gain(dbfs_delta)


def split_audio_file_on_silence(file_path: str,
                                output_directory: str,
                                min_silence_length: int,
                                threshold: int,
                                added_silence: int,
                                file_index: int) -> None:
    """
    Splits an AudioSegment into sub-segments based on silence detected by pydub.
    :param file_path: file path of the audio file to split
    :param output_directory: path to directory in which to write output files
    :param min_silence_length: the minimum length (in ms) of silence that indicates a break
    :param threshold: the level below the norm (in dBFS) to consider silence
    :param added_silence: silence to be added to the beginning and end of each split utterance
    :param file_index: the number of the file in the directory (recursive) to mark each sub-utterance with.
    """
    audio = AudioSegment(file_path)
    segments = split_on_silence(audio_segment=audio,
                                min_silence_len=min_silence_length,
                                silence_thresh=-threshold)
    silence = AudioSegment.silent(duration=added_silence)
    for segment_index, segment in enumerate(segments):
        audio_segment = silence + segment + silence
        normalised_segment = match_target_amplitude(audio_segment, -20)
        export_file_name = f"_file_{file_index}-part_{segment_index}.wav"
        print(f"Exporting {export_file_name}")
        normalised_segment.export(Path(output_directory, export_file_name))


def main() -> None:
    parser = ArgumentParser(description="Splits a directory of audio (.wav) files into short segments")
    parser.add_argument("-i", "--input_dir",
                        help="Directory containing audio to be split",
                        type=str,
                        required=True)
    parser.add_argument("-o", "--output_dir",
                        help="Directory to output the split audio",
                        type=str,
                        required=True)
    parser.add_argument("-s", "--silence_length",
                        help="Minimum length of silence in milliseconds",
                        type=int,
                        default=200)
    parser.add_argument("-t", "--threshold",
                        help="Threshold below norm to consider silence in dBFS (positive integer)",
                        type=int,
                        default=16)
    parser.add_argument("-a", "--added_silence",
                        help="Add silence to beginning and end of segments, in milliseconds",
                        type=int,
                        default=100)

    arguments = parser.parse_args()
    all_audio_files = find_all_files_in_dir_by_extensions(arguments.input_dir, {"wav"})

    for index, file_path in enumerate(all_audio_files):
        split_audio_file_on_silence(file_path=file_path,
                                    output_directory=arguments.output_dir,
                                    min_silence_length=arguments.silence_length,
                                    threshold=arguments.threshold,
                                    added_silence=arguments.added_silence,
                                    file_index=index)


if __name__ == "__main__":
    main()



import sys
from kaldi_helpers.script_utilities import *
from _pytest.capture import CaptureFixture

"""
Test script for validating trs to json conversion methods. 

@author Nicholas Lambourne 
"""

TEST_FILES_BASE_DIR = os.path.join(".", "test", "testfiles")
EXAMPLE_JSON_DATA = [
    {"transcript": "Comment t'appelles tu?"},
    {"transcript": "Je m'appelle François."},
    {"transcript": "Est-ce tu a une livre préférér."},
    {"transcript": "Oui, j'adore L'histoire secrète par Donna Tartt."},
    {"transcript": "Vraiment? Je n'ai jamais lu ça."},
]


def test_load_json_file():
    json_data = load_json_file(os.path.join(TEST_FILES_BASE_DIR, "test_json_load.json"))
    assert json_data[0]["transcript"] == "Comment t'appelles tu?"


def test_write_data_to_json_file_path():
    write_data_to_json_file(EXAMPLE_JSON_DATA, "test_file.json")
    with open("test_file.json", "r") as test_file:
        assert json.load(test_file) == EXAMPLE_JSON_DATA
    os.remove('test_file.json')


def test_write_data_to_json_stdout(capsys: CaptureFixture):
    write_data_to_json_file(EXAMPLE_JSON_DATA, sys.stdout)
    out, _ = capsys.readouterr()
    assert out == json.dumps(EXAMPLE_JSON_DATA, indent=4) + "\n"


def test_find_first_file_by_extension() -> None:
    all_files_in_dir = list(glob.glob(os.path.join(TEST_FILES_BASE_DIR, "**"), recursive=True))
    all_files_in_dir.sort()

    first_test: str = find_first_file_by_extension(all_files_in_dir, list(["*.rtf"]))
    assert os.path.split(first_test)[1].endswith(".rtf")
    assert os.path.basename(first_test) == "test.rtf"

    second_test: str = find_first_file_by_extension(all_files_in_dir, list(["*.txt"]))
    assert os.path.split(second_test)[1].endswith(".txt")
    assert os.path.basename(second_test) != "test.txt"
    assert os.path.basename(second_test) == "howdy.txt"

    third_test: str = find_first_file_by_extension(all_files_in_dir, list(["*.py", "*.xlsx"]))
    assert os.path.split(third_test)[1].endswith(".xlsx")
    assert os.path.basename(third_test) != "howdy.xlsx"
    assert os.path.basename(third_test) == "charm.xlsx"


#!/usr/bin/python3

"""
Converts audio to 16 bit 16k mono WAV

Copyright: University of Queensland, 2019
Contributors:
              Scott Heath - (The University of Queensland, 2017)
              Oliver Adams - (The University of Melbourne, 2018)
"""

import argparse
import glob
import os
import subprocess
import threading
from multiprocessing.dummy import Pool
from shutil import move
from typing import Set, Tuple
from kaldi_helpers.script_utilities import find_files_by_extensions
from kaldi_helpers.script_utilities.globals import SOX_PATH


def join_norm(p1, p2):
    tmp = os.path.join(os.path.normpath(p1), os.path.normpath(p2))
    return os.path.normpath(tmp)


def process_item(sox_arguments: Tuple[int, str, threading.Lock, Set[str], str]):
    index, input_audio, lock, temporary_directories, parent_temporary_directory = sox_arguments

    input_name = os.path.normpath(input_audio)

    file_directory, file_name = os.path.split(input_audio)
    base_directory, ext = os.path.splitext(file_name)
    output_directory = os.path.join(file_directory, parent_temporary_directory)

    with lock:  # Avoids race condition
        temporary_directories.add(output_directory)
        if not os.path.exists(output_directory):
            os.makedirs(output_directory)

    temporary_file_name = join_norm(output_directory, "%s.%s" % (base_directory, "wav"))

    if not os.path.exists(temporary_file_name):
        sox_arguments = [SOX_PATH, input_name, "-b", "16", "-c", "1", "-r", "44.1k", "-t", "wav", temporary_file_name]
        subprocess.call(sox_arguments)
    return temporary_file_name


def main() -> None:
    parser = argparse.ArgumentParser(description="This script will silence a wave file based on annotations in "
                                                 "an Elan tier ")
    parser.add_argument('-c', '--corpus',
                        help='Directory of audio and eaf files',
                        type=str,
                        default='../input/data')
    parser.add_argument('-o', '--overwrite',
                        help='Write over existing files',
                        action="store_true",
                        default=True)
    args = parser.parse_args()

    base_directory = args.corpus
    audio_extensions = {"*.wav"}
    parent_temporary_directory = "tmp"

    all_files_in_dir = glob.glob(os.path.join(base_directory, "**"), recursive=True)
    input_audio = find_files_by_extensions(all_files_in_dir, audio_extensions)
    process_lock = threading.Lock()
    temporary_directories = set()

    map_arguments = [(index, audio_path, process_lock, temporary_directories, parent_temporary_directory)
                     for index, audio_path in enumerate(input_audio)]

    # Multi-Threaded Audio Re-sampling
    with Pool() as pool:
        outputs = pool.map(process_item, map_arguments)

        if args.overwrite:
            # Replace original files
            for audio_file in outputs:
                file_name = os.path.basename(audio_file)
                parent_directory = os.path.dirname(os.path.dirname(audio_file))
                move(audio_file, os.path.join(parent_directory, file_name))
            # Clean up tmp folders
            for d in temporary_directories:
                os.rmdir(d)


if __name__ == "__main__":
    main()



from kaldi_helpers.json_to_kaldi import *

SCRIPT_PATH: str = os.path.join(".", "kaldi_helpers", "json_to_kaldi.py")

def test_json_to_kaldi1():
    
    input_json: str = os.path.join(".", "test", "testfiles", "example.json")
    output_dir: str = os.path.join(".", "data")
    command: List[str] = ["python", f"{SCRIPT_PATH}",
               "--input_json", f"{input_json}",
               "--output_folder", f"{output_dir}"]
    command: str = subprocess.list2cmdline(command)
    os.system(command)

    required_files: Set[str] = {"corpus.txt", "segments", "spk2gender", "text", "utt2spk", "wav.scp"}
    for folder in os.listdir(output_dir):
        folder: str = os.path.join(output_dir, folder)
        if os.path.isdir(folder):
            files_present: Set[str] = set(os.listdir(folder))
            assert len(required_files) <= len(files_present)
            assert required_files.issubset(files_present)
    shutil.rmtree(output_dir)


def test_json_to_kaldi2():
    input_json = os.path.join(".", "test", "testfiles", "does_not_exist.json")
    output_dir = os.path.join(".", "data")
    command = ["python", f"{SCRIPT_PATH}",
               "--input_json", f"{input_json}",
               "--output_folder", f"{output_dir}"
                                  "--silence_markers"]
    command = subprocess.list2cmdline(command)

    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    while process.poll() is None:
        process.stdout.readline()  # give output_scripts from your execution/your own message
    command_result = process.wait()  # catch return code
    assert command_result != 0 # assert failure




"""
Test script for validating trs to json conversion methods. 

@author Aninda Saha
"""

from kaldi_helpers.input_scripts.trs_to_json import *

SCRIPT_PATH = os.path.join(".", "kaldi_helpers", "trs_to_json.py")


def test_find_files_by_extension() -> None:

    all_files_in_directory: List[str] = set(glob.glob(os.path.join(TEST_FILES_BASE_DIR, "**"), recursive=True))

    first_test: List[str] = find_files_by_extensions(all_files_in_directory, set(["*.xlsx"]))
    files = set([os.path.split(i)[1] for i in first_test])
    assert len(first_test) == 3 # Number of .xlsx files stored in testfiles folder
    for file in first_test:
        assert file.endswith(".xlsx")
    assert {"python.xlsx", "test.xlsx", "charm.xlsx"}.issubset(files)

    second_test: List[str] = find_files_by_extensions(all_files_in_directory, set(["*.py"]))
    files = set([os.path.split(i)[1] for i in second_test])
    assert len(second_test) == 1 # Number of .py files stored in testfiles folder
    for file in second_test:
        assert file.endswith(".py")
    assert {"test.py"}.issubset(files)

    third_test: List[str] = find_files_by_extensions(all_files_in_directory, set(["*.py", "*.txt"]))
    files = set([os.path.split(i)[1] for i in third_test])
    assert len(third_test) == 3 # Total number of .py and .txt files stored in testfiles folder
    for file in third_test:
        assert (file.endswith(".py") or file.endswith(".txt"))
    assert {"test.py", "howdy.txt", "test.txt"}.issubset(files)


def test_conditional_log() -> None:

    sys.stderr: TextIOWrapper = open('err.txt', 'w')
    test_str1: str = "test"
    conditional_log(condition=True, text=test_str1);
    with open("err.txt", "r") as f:
        assert test_str1 == f.read()
        f.close()

    sys.stderr: TextIOWrapper = open('err.txt', 'w')
    test_str2: str = "Kaldi is a fun project\nASR is a cool tech\n"
    conditional_log(condition=True, text=test_str2);
    with open("err.txt", "r") as f:
        assert test_str2 == f.read()
        f.close()

    sys.stderr = sys.__stderr__
    os.remove('err.txt')


def test_process_trs_file():

    all_files_in_directory: Set[str] = set(glob.glob(os.path.join(TEST_FILES_BASE_DIR, "*.trs"),
                                           recursive=True))
    for file_name in all_files_in_directory:
        with open(file_name) as f:
            contents: str = f.read()
            count: int = sum(1 for match in regex.finditer(r"\bSync\b", contents, flags=regex.IGNORECASE))
            utterances: List[Dict[str, Union[str, float]]] = process_trs(file_name, False)
            assert count == len(utterances)


def test_process_turn():

    all_files_in_directory: Set[str] = set(glob.glob(os.path.join(TEST_FILES_BASE_DIR, "*.trs"),
                                                     recursive=True))

    for file_name in all_files_in_directory:
        with open(file_name) as f:
            contents: str = f.read()
            contents_list: List[str] = contents.split('\n')
            matched_lines: int = [contents_list.index(line) for line in contents_list if
                             ("Turn" in line) and (line != '</Turn>')] + [len(contents_list)]
            tree: ET.ElementTree = ET.parse(file_name)
            root: ET.Element = tree.getroot()
            wave_name: str = root.attrib['audio_filename'] + ".wav"
            turn_nodes: List[ET.Element] = tree.findall(".//Turn")

            for i in range(len(turn_nodes)):
                turn_contents: str = "\n".join(contents_list[matched_lines[i]:matched_lines[i+1]])
                count: int = sum(1 for match in regex.finditer(r"\bSync\b", turn_contents, flags=regex.IGNORECASE))
                utterances_in_turn: List[Dict[str, Union[str, float]]] = process_turn(wave_name, turn_nodes[i], tree)
                assert count == len(utterances_in_turn)


def test_trs_to_json():

    all_files_in_directory: Set[str] = set(glob.glob(os.path.join(TEST_FILES_BASE_DIR, "*.trs"),
                                                     recursive=True))
    utterances: List[Dict[str, Union[str, float]]] = []
    for file_name in all_files_in_directory:
        utterances = utterances + process_trs(file_name, False)

    command: List[str] = ["python", SCRIPT_PATH, "--input_dir", TEST_FILES_BASE_DIR]
    result: subprocess.CompletedProcess = subprocess.run(command, check=True)
    assert result.returncode == 0

    parent_directory_name, base_directory_name = os.path.split(TEST_FILES_BASE_DIR)
    json_name: str = os.path.join(TEST_FILES_BASE_DIR, base_directory_name+".json")
    with open(json_name) as f:
        contents: List[Dict[str, Union[str, float]]] = json.loads(f.read())
    os.remove(json_name)

    assert (len(contents) == len(utterances))
    assert contents == utterances




#!/usr/bin/python3

"""
This is script automatically builds a pronunciation (word->sounds) dictionary

Copyright: University of Queensland, 2019
Contributors:

"""

import argparse
import sys
from typing import List, Tuple, Set, TextIO


def extract_words(input_file_name: str) -> List[str]:
    """
    Extracts a list of words from a word list. Expects one word per line.
    :param input_file_name: the path of the file to extract the words from
    :return: a list of words
    """
    input_tokens = []
    with open(input_file_name, "r", encoding='utf-8') as input_file:
        for line in input_file.readlines():
            token = line.strip()
            if len(token) > 0:
                input_tokens.append(token)
    return input_tokens


def extract_sound_mappings(config_file_name: str) -> List[Tuple[str, str]]:
    """
    Extract a list of mappings from characters/symbols to sounds.
    :param config_file_name: the path to the file to extract the mappings from
    :return: a list of tuples consisting containing (symbol, sound_equivalent)
    """
    sound_mappings = []
    with open(config_file_name, "r", encoding='utf-8') as config_file:
        for line in config_file.readlines():
            if line[0] == '#':
                continue

            mapping = list(filter(None, line.strip().split(' ', 1)))

            if len(mapping) > 1:
                sound_mappings.append((mapping[0], mapping[1]))
    return sound_mappings


def generate_sound_mapping(word: str,
                           sound_map: List[Tuple[str, str]],
                           output_file: TextIO,
                           missing_characters: Set[str]) -> None:
    """
    Writes the pronunciation mapping for a particular word to the given output file.
    :param word: the word to generate a pronunciation mapping for
    :param sound_map: the sound map to base the pronunciation on
    :param output_file: the file to write the mapping to
    :param missing_characters: a list to store any unrecognised characters in
    """
    current_index = 0
    res = [word]
    token_lower = word.lower()

    while current_index < len(token_lower):
        found = False
        for maps in sound_map:
            if token_lower.find(maps[0], current_index) == current_index:
                found = True
                res.append(maps[1])
                current_index += len(maps[0])
                break

        if not found:
            # unknown sound
            res.append('(' + token_lower[current_index] + ')')
            missing_characters.add(token_lower[current_index])
            current_index += 1

    output_file.write(' '.join(res) + '\n')


def generate_pronunciation_dictionary(word_list: str,
                                      pronunciation_dictionary: str,
                                      config_file: str) -> None:
    """
    Creates a dictionary of pronunciations based on the provided word list and sound rules.
    :param word_list: the file path to the list of words to add to the pronunciation dictionary
    :param pronunciation_dictionary: the path to the file to write the pronunciation dictionary
    :param config_file: the path to the file with the symbol -> sound mapping
    """
    words = extract_words(word_list)
    sound_map = extract_sound_mappings(config_file)
    sound_map.sort(key=lambda x: len(x[0]), reverse=True)  # Sort by length of sound map

    missing_characters = set()

    with open(pronunciation_dictionary, "w", encoding='utf-8') as output_file:
        output_file.write('!SIL sil\n')
        output_file.write('<UNK> spn\n')
        for word in words:
            generate_sound_mapping(word=word,
                                   sound_map=sound_map,
                                   output_file=output_file,
                                   missing_characters=missing_characters)

    for character in missing_characters:
        print(f"Unexpected character: {character}", file=sys.stderr)

    print("Done", file=sys.stderr)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--infile",
                        type=str,
                        required=True,
                        help="")
    parser.add_argument("-o", "--outfile",
                        type=str,
                        required=True,
                        help="name of the output_scripts file")
    parser.add_argument("-c", "--config",
                        type=str,
                        required=True,
                        help="configuration file with one letter/symbol -> sound mapping in each line")
    arguments = parser.parse_args()

    generate_pronunciation_dictionary(word_list=arguments.infile,
                                      pronunciation_dictionary=arguments.outfile,
                                      config_file=arguments.config)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

"""
Given a json file with transcript information these tools can perform
- manipulations including generating word lists or filtering output_scripts to
- exclude english/punctuation.
- Optionally provide the output_scripts json file name with -j

Usage: python3 clean_json.py [-h] [--i INFILE] [--o OUTFILE] [-r] [-u]

Copyright: University of Queensland, 2019
Contributors:
              Josh Arnold - (The University of Queensland, 2017)
              Ben Foley - (The University of Queensland, 2018)
              Nicholas Lambourne - (The University of Queensland, 2019)
"""

import re
import string
import sys
import nltk
from argparse import ArgumentParser
from langid.langid import LanguageIdentifier, model
from nltk.corpus import words
from typing import Dict, List, Set
from kaldi_helpers.script_utilities import load_json_file, write_data_to_json_file


def get_english_words() -> Set[str]:
    """
    Gets a list of English words from the nltk corpora (~235k words).
    N.B: will download the word list if not already available (~740kB), requires internet.
    :return: a set containing the English words
    """
    nltk.download("words")  # Will only download if not locally available.
    return set(words.words())


def clean_utterance(utterance: Dict[str, str],
                    remove_english: bool = False,
                    english_words: set = None,
                    punctuation: str = None,
                    special_cases: List[str] = None) -> (List[str], int):
    """
    Takes an utterance and cleans it based on the rules established by the provided parameters.
    :param utterance: a dictionary with a "transcript" key-value pair.
    :param remove_english: whether or not to remove English dirty_words.
    :param english_words: a list of english dirty_words to remove from the transcript (we suggest the nltk dirty_words corpora).
    :param punctuation: list of punctuation symbols to remove from the transcript.
    :param special_cases: a list of dirty_words to always remove from the output_scripts.
    :return: a tuple with a list of 'cleaned' dirty_words and a number representing the number of English dirty_words to remove.
    """
    translation_tags = {"@eng@", "<ind:", "<eng:"}
    utterance_string = utterance.get("transcript").lower()
    dirty_words = utterance_string.split()
    clean_words = []
    english_word_count = 0
    for word in dirty_words:
        if special_cases and word in special_cases:
            continue
        if word in translation_tags:  # Translations / ignore
            return [], 0
        # If a word contains a digit, throw out whole utterance
        if bool(re.search(r"\d", word)) and not word.isdigit():
            return [], 0
        if punctuation:
            for mark in punctuation:
                word = word.replace(mark, "")
        if remove_english and len(word) > 3 and word in english_words:
            english_word_count += 1
            continue
        clean_words.append(word)
    return clean_words, english_word_count


def is_valid_utterance(clean_words: List[str],
                       english_word_count: int,
                       remove_english: bool,
                       use_langid: bool,
                       langid_identifier: LanguageIdentifier) -> bool:
    """
    Determines whether a cleaned utterance (list of words) is valid based on the provided parameters.
    :param clean_words: a list of clean word strings.
    :param english_word_count: the number of english words removed from the string during cleaning.
    :param remove_english: whether or not to remove english words.
    :param use_langid: whether or not to use the langid library to determine if a word is English.
    :param langid_identifier: language identifier object to use with langid library.
    :return: True if utterance is valid, false otherwise.
    """
    # Exclude utterance if empty after cleaning
    cleaned_transcription = " ".join(clean_words).strip()
    if cleaned_transcription == "":
        return False

    # Exclude utterance if > 10% english
    if remove_english and len(clean_words) > 0 and english_word_count / len(clean_words) > 0.1:
        # print(round(english_word_count / len(clean_words)), trans, file=sys.stderr)
        return False

    # Exclude utterance if langid thinks its english
    if remove_english and use_langid:
        lang, prob = langid_identifier.classify(cleaned_transcription)
        if lang == "en" and prob > 0.5:
            return False
    return True


def clean_json_data(json_data: List[Dict[str, str]],
                    remove_english: bool = False,
                    use_langid: bool = False) -> List[Dict[str, str]]:
    """
    Clean a list of utterances (Python dictionaries) based on the given parameters.
    :param json_data: list of Python dictionaries, each must have a 'transcription' key-value.
    :param remove_english: whether or not to remove English from the utterances.
    :param use_langid: whether or not to use the langid library to identify English to remove.
    :return: cleaned list of utterances (list of dictionaries).
    """
    punctuation_to_remove = string.punctuation + "…’“–”‘°"
    special_cases = ["<silence>"]  # Any words you want to ignore
    langid_identifier = None

    if remove_english:
        english_words = get_english_words()  # pre-load English corpus
        if use_langid:
            langid_identifier = LanguageIdentifier.from_modelstring(model,
                                                                    norm_probs=True)
    else:
        english_words = set()

    cleaned_data = []
    for utterance in json_data:
        clean_words, english_word_count = clean_utterance(utterance=utterance,
                                                          remove_english=remove_english,
                                                          english_words=english_words,
                                                          punctuation=punctuation_to_remove,
                                                          special_cases=special_cases)

        if is_valid_utterance(clean_words,
                              english_word_count,
                              remove_english,
                              use_langid,
                              langid_identifier):
            cleaned_transcript = " ".join(clean_words).strip()
            utterance["transcript"] = cleaned_transcript
            cleaned_data.append(utterance)

    return cleaned_data


def main() -> None:
    """
    Run the entire clean_json process as a command line utility.

    Usage: python3 clean_json.py [--i INFILE] [--o OUTFILE] [-r] [-u]
    """
    parser: ArgumentParser = ArgumentParser()
    parser.add_argument("-i", "--infile",
                        type=str,
                        help="The path to the dirty json file to clean.",
                        required=True)
    parser.add_argument("-o", "--outfile",
                        type=str,
                        help="The path to the clean json file to write to",
                        required=True)
    parser.add_argument("-r", "--remove_eng",
                        help="Remove english like utterances",
                        action="store_true")
    parser.add_argument("-u", "--use_lang_id",
                        help="Use langid library to detect English",
                        action="store_true")

    arguments = parser.parse_args()
    dirty_json_data: List[Dict[str, str]] = load_json_file(arguments.infile)
    outfile = arguments.outfile if arguments.outfile else sys.stdout

    print(f"Filtering dirty json data {arguments.infile}...")

    filtered_data = clean_json_data(json_data=dirty_json_data,
                                    remove_english=arguments.remove_eng,
                                    use_langid=arguments.use_lang_id)

    write_data_to_json_file(data=list(filtered_data),
                            output=outfile)

    print(f"Finished! Wrote {str(len(filtered_data))} transcriptions.")


if __name__ == "__main__":
    main()



import os

# Used by test scripts to tap into a folder of test files
TEST_FILES_BASE_DIR = os.path.join(".", "test", "testfiles")
DEFAULT_DATA_DIRECTORY = os.path.join(".", "resources", "corpora", "abui_toy_corpus", "data")

# Used by resample_audio.py
AUDIO_EXTENSIONS = ["*.wav"]
TEMPORARY_DIRECTORY = "tmp"
SOX_PATH = os.path.join("/", "usr", "bin", "sox")



import os
from kaldi_helpers.make_wordlist import *
from typing import List


def test_save_word_list() -> None:
    file_name: str = "test.txt"
    word_list: List[str] = [
        "hello",
        "world",
        "i",
        "love",
        "test"
    ]
    save_word_list(word_list, file_name)
    with open(file_name, "r") as file:
        assert file.read() == "\n".join(word_list) + "\n"
    os.remove("test.txt")  # Clean up


def test_extract_word_list() -> None:
    json_data = [
        {"transcript": "Hello world I love test"},
        {"transcript": "I love test too"}
    ]
    result = extract_word_list(json_data)
    assert result == ["Hello", "I", "love", "test", "too", "world"]





'''
Copyright: University of Queensland, 2019
Contributors:
             Nicholas Lambourne - (University of Queensland, 2018)
'''

from setuptools import setup, find_packages


requirements = [
    'pytest',
    'pympi-ling',
    'numpy',
    'langid',
    'nltk',
    'praatio',
    'pydub',
]

package_data = {
    'kaldi_helpers/inference_scripts': ['*']
}

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='kaldi_helpers',
    version='0.22',
    packages=find_packages(),
    url='https://github.com/CoEDL/kaldi_helpers',
    install_requires=requirements,
    include_package_data=True,
    license='',
    author='CoEDL',
    author_email='n.lambourne@uq.edu.au',
    description='Scripts for preparing language data for use with Kaldi ASR',
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Topic :: Text Processing :: Linguistic",
        "Operating System :: OS Independent",
    ],
    entry_points={
    },
)



#!/usr/bin/python3

"""
Parse json file and extract transcription information which are then processed and output_scripts in the desired Kaldi format.
The output_scripts files will be stored in two separate folders training and testing inside the specified output_scripts directory.

training : 
    corpus.txt, text, segments, wav.scp, utt2spk, spk2utt
testing : 
    corpus.txt, text, segments, wav.scp, utt2spk, spk2utt
            
The training folder is for the model creation using Kaldi, whereas the testing folder is used for verifying the 
reliability of the model.

Copyright: University of Queensland, 2019
Contributors:
              Scott Heath - (University of Queensland, 2017)
              Nicholas Lambourne - (University of Queensland, 2019)
"""

import argparse
import glob
import json
import os
import re
import uuid
from typing import Dict, List
from _io import TextIOWrapper


def extract_additional_corpora(file_name: str, kaldi_corpus: str) -> None:
    """
    Takes a text file, extracts all sentences and writes them to the corpus file.
    :param file_name: the path to a plaintext file to extract additional sentences/lines from
    :param kaldi_corpus: the path to kaldi corpus.txt file created by json_to_kaldi.py.
    """
    if os.path.exists(kaldi_corpus):
        append_write = 'a'  # append if already exists
    else:
        append_write = 'w'  # make a new file if not
    with open(kaldi_corpus, append_write) as kaldi_corpus_file:
        if os.path.exists(file_name):
            print(f"Extracting corpus examples from: {file_name}")
            with open(file_name, "r", encoding="utf-8", ) as file_:
                for line in file_.readlines():
                    kaldi_corpus_file.writelines(re.sub(r"[^a-zA-Z0-9\s]", "", line))
        else:
            print(f"Provided additional text corpus file path invalid: {file_name}")


def clean_corpus_file(corpus_file_path: str) -> List[str]:
    """
    Opens the given file, removes punctuation and returns a list of cleaned lines.
    :param corpus_file_path: file path to additional corpus examples
    :return: a list of cleaned corpus examples
    """
    examples = []
    with open(corpus_file_path, "r") as file_:
        for line in file_.readlines():
            clean_line = re.sub(r"[^a-zA-Z0-9\s]", "", line)
            examples.append(clean_line)
    return examples


class KaldiInput:
    """
    Class to store information for the training and testing data sets.     
    """

    def __init__(self, output_folder: str) -> None:

        if not os.path.exists(output_folder):
            os.makedirs(output_folder)

        self.speakers: Dict[str, str] = {}
        self.recordings: Dict[str, str] = {}
        self.utterances: Dict[str, str] = {}

        self.segments_list: List[str] = []
        self.transcripts_list: List[str] = []
        self.speakers_list: List[str] = []
        self.recordings_list: List[str] = []
        self.utt2spk_list: List[str] = []
        self.corpus_list: List[str] = []

        self.segments_file: TextIOWrapper = open(f"{output_folder}/segments", "w", encoding="utf-8")
        self.transcripts_file: TextIOWrapper = open(f"{output_folder}/text", "w", encoding="utf-8")
        self.speakers_file: TextIOWrapper = open(f"{output_folder}/spk2gender", "w", encoding="utf-8")
        self.recordings_file: TextIOWrapper = open(f"{output_folder}/wav.scp", "w", encoding="utf-8")
        self.utt2spk_file: TextIOWrapper = open(f"{output_folder}/utt2spk", "w", encoding="utf-8")
        self.corpus_file: TextIOWrapper = open(f"{output_folder}/corpus.txt", "w", encoding="utf-8")

    def add_speaker(self, speaker_id: str) -> str:
        """
        Adds a speaker element if it is not already present.
        
        :param speaker_id: speaker id - could be a name of a uuid code
        :return: returns the correctly formatted speaker id 
        """
        if speaker_id not in self.speakers:
            self.speakers[speaker_id] = str(uuid.uuid4())  # create speaker id
            self.speakers_list.append(f"{self.speakers[speaker_id]} \n")  # writing gender
        return self.speakers[speaker_id]

    def add_recording(self, audio_file: str) -> str:
        """
        Adds an audio file it is not already present.
        
        :param audio_file: name of audio file 
        :return: returns a correctly formatted audio file description
        """
        if audio_file not in self.recordings:
            self.recordings[audio_file] = str(uuid.uuid4())  # Create recording id
            self.recordings_list.append(f"{self.recordings[audio_file]} ./{audio_file}\n")
        return self.recordings[audio_file]

    def add(self, recording_id: str,
            speaker_id: str,
            utterance_id: str,
            start_ms: int,
            stop_ms: int,
            transcript: str,
            silence_markers: bool) -> None:
        """
        Appends new items to the transcripts, segments, utt2spk and corpus lists.
        
        :param recording_id: id for the recording file
        :param speaker_id: id for the speaker who uttered the phrase
        :param utterance_id: unique id for the uttered phrase
        :param start_ms: start of the uttered phrase
        :param stop_ms: stop time of the uttered phrase
        :param transcript: the uttered phrase
        :param silence_markers: boolean condition indicating whether to include silence markers
        """
        if silence_markers:
            self.transcripts_list.append(f"{utterance_id} !SIL {transcript} !SIL\n")
        else:
            self.transcripts_list.append(f"{utterance_id} {transcript} \n")
        self.segments_list.append(f"{utterance_id} {recording_id} {start_ms/1000.0} {stop_ms/1000.0}\n")
        self.utt2spk_list.append(f"{utterance_id} {speaker_id}\n")
        self.corpus_list.append(f"{transcript}\n")

    def write_and_close(self) -> None:
        """
        After parsing the json file and populating the segments, transcripts, speakers, recordings, utt2spk and corpus 
        lists with data, this function performs the final write to their respective files.
        """

        self.segments_list.sort()
        self.segments_file.write("".join(self.segments_list))
        self.segments_file.close()

        self.transcripts_list.sort()
        self.transcripts_file.write("".join(self.transcripts_list))
        self.transcripts_file.close()

        self.speakers_list.sort()
        self.speakers_file.write("".join(self.speakers_list))
        self.speakers_file.close()

        self.recordings_list.sort()
        self.recordings_file.write("".join(self.recordings_list))
        self.recordings_file.close()

        self.utt2spk_list.sort()
        self.utt2spk_file.write("".join(self.utt2spk_list))
        self.utt2spk_file.close()

        self.corpus_list.sort()
        self.corpus_file.write("".join(self.corpus_list))
        self.corpus_file.close()


def extract_transcript(input_set: KaldiInput,
                       json_transcript: dict,
                       silence_markers: bool) -> None:
    """
    Extract a single transcript from json and add its contents to the given output set.
    :param input_set: the set to add the data from the transcript to (e.g. testing or training data)
    :param json_transcript:
    :param silence_markers: boolean condition indicating whether to include silence markers
    """
    transcript: str = json_transcript.get("transcript", "")
    start_ms: int = json_transcript.get("start_ms", 0)
    stop_ms: int = json_transcript.get("stop_ms", 0)

    # Speaker ID is not available in textgrid files
    if "speaker_id" in json_transcript:
        speaker_id: str = json_transcript.get("speaker_id", "")
    else:
        speaker_id: str = str(uuid.uuid4())

    audio_file: str = json_transcript.get("audio_file_name", "").replace("\\", "/")

    speaker_id = input_set.add_speaker(speaker_id)  # add speaker id
    recording_id: str = input_set.add_recording(audio_file)  # add audio file name
    utterance_id: str = speaker_id + "-" + str(uuid.uuid4())  # add utterance id
    input_set.add(recording_id,
                  speaker_id,
                  utterance_id,
                  start_ms,
                  stop_ms,
                  transcript,
                  silence_markers)


def create_kaldi_structure(input_json: str,
                           output_folder: str,
                           silence_markers: bool,
                           text_corpus: str,
                           corpus_file: str) -> None:
    """
    Create a full Kaldi input structure based upon a json list of transcriptions and an optional
    text corpus.
    :param input_json: the path to a json file with a list of transcriptions
    :param output_folder: the folder in which to create the kaldi file stucture
    :param silence_markers: boolean condition indicating whether to include silence markers
    :param text_corpus: path to the directory containing the text corpus
    :param corpus_file: the path to the file to write all corpus examples to
    """
    testing_input = KaldiInput(output_folder=f"{output_folder}/testing")
    training_input = KaldiInput(output_folder=f"{output_folder}/training")

    try:
        with open(input_json, "r") as input_file:
            json_transcripts: str = json.loads(input_file.read())
    except FileNotFoundError:
        print(f"JSON file could not be found: {input_json}")
        return

    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    for i, json_transcript in enumerate(json_transcripts):
        if i % 10 == 0:
            extract_transcript(input_set=testing_input,
                               json_transcript=json_transcript,
                               silence_markers=silence_markers)
        else:
            extract_transcript(input_set=training_input,
                               json_transcript=json_transcript,
                               silence_markers=silence_markers)

    if text_corpus:
        text_corpus_directory = text_corpus
        print(f"Using additional text corpus at {text_corpus_directory}")
        all_files_in_dir = set(glob.glob(os.path.join(text_corpus_directory, "**"), recursive=True))
        only_text = []
        for file_ in all_files_in_dir:
            file_name, extension = os.path.splitext(file_)
            if extension == ".txt":
                only_text.append(file_)
        for corpora_file in only_text:
            extract_additional_corpora(corpora_file, corpus_file)
            training_input.corpus_list.extend(clean_corpus_file(corpora_file))
    else:
        print("No additional text corpus provided.")

    testing_input.write_and_close()
    training_input.write_and_close()


def main() -> None:
    """ 
    Run the entire json_to_kaldi.py as a command line utility. 
    
    Usage: python3 json_to_kaldi.py -i INPUT_JSON -o OUTPUT_FOLDER [-s] [-t TEXT_CORPUS] [-c CORPUS_FILE]
    """
    parser = argparse.ArgumentParser(description="Convert json from stdin to Kaldi input_scripts files "
                                                 "(in output_scripts-folder).")
    parser.add_argument("-i", "--input_json",
                        type=str,
                        help="The input_scripts json file",
                        required=True)
    parser.add_argument("-o", "--output_folder",
                        type=str,
                        help="The output_scripts folder",
                        default=os.path.join(".", "data"))
    parser.add_argument("-s", "--silence_markers",
                        action="store_true",
                        help="The input_scripts json file",
                        required=False)
    parser.add_argument("-t", "--text_corpus",
                        help="File path to a folder of text-only corpus files to include in corpus.txt.",
                        required=False)
    parser.add_argument("-c", "--corpus_file",
                        type=str,
                        help="Path to the corpus.txt file to write text examples to",
                        required=False)
    arguments = parser.parse_args()

    create_kaldi_structure(input_json=arguments.input_json,
                           output_folder=arguments.output_folder,
                           silence_markers=arguments.silence_markers,
                           text_corpus=arguments.text_corpus,
                           corpus_file=arguments.corpus_file)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

"""
Get all files in the repository can use recursive atm as long as we don't need numpy
pass in corpus path throw an error if matching file wav isn"t found in the corpus directory

Usage: python3 elan_to_json.py [-h] [-i INPUT_DIR] [-o OUTPUT_DIR] [-t TIER] [-j OUTPUT_JSON]

Copyright: University of Queensland, 2019
Contributors:
              Nicholas Lambourne - (The University of Queensland, 2018)
"""

import glob
import sys
import os
import argparse
from pympi.Elan import Eaf
from typing import List
from kaldi_helpers.script_utilities import find_files_by_extensions
from kaldi_helpers.script_utilities import write_data_to_json_file


def process_eaf(input_elan_file: str, tier_name: str) -> List[dict]:
    """
    Method to process a particular tier in an eaf file (ELAN Annotation Format). It stores the transcriptions in the 
    following format:
                    {'speaker_id': <speaker_id>,
                    'audio_file_name': <file_name>,
                    'transcript': <transcription_label>,
                    'start_ms': <start_time_in_milliseconds>,
                    'stop_ms': <stop_time_in_milliseconds>}
                    
    :param input_elan_file: name of input_scripts elan file
    :param tier_name: name of the elan tier to process. these tiers are nodes from the tree structure in the .eaf file.
    :return: a list of dictionaries, where each dictionary is an annotation
    """
    # Get paths to files
    input_directory, full_file_name = os.path.split(input_elan_file)
    file_name, extension = os.path.splitext(full_file_name)

    input_eaf = Eaf(input_elan_file)

    # Look for wav file matching the eaf file in same directory
    if os.path.isfile(os.path.join(input_directory, file_name + ".wav")):
        print("WAV file found for " + file_name, file=sys.stderr)
    else:
        raise ValueError(f"WAV file not found for {full_file_name}. "
                         f"Please put it next to the eaf file in {input_directory}.")

    # Get annotations and parameters (things like speaker id) on the target tier
    annotations = sorted(input_eaf.get_annotation_data_for_tier(tier_name))
    parameters = input_eaf.get_parameters_for_tier(tier_name)
    speaker_id = parameters.get("PARTICIPANT", "")

    annotations_data = []

    for annotation in annotations:
        start = annotation[0]
        end = annotation[1]
        annotation = annotation[2]

        # print("processing annotation: " + annotation, start, end)
        obj = {
            "audio_file_name": f"{file_name}.wav",
            "transcript": annotation,
            "start_ms": start,
            "stop_ms": end
        }
        if "PARTICIPANT" in parameters:
            obj["speaker_id"] = speaker_id
        annotations_data.append(obj)

    return annotations_data


def main():

    """ 
    Run the entire elan_to_json.py as a command line utility. It extracts information on speaker, audio file, 
    transcription etc. from the given tier of the specified .eaf file. 
    
    Usage: python3 elan_to_json.py [-h] [-i INPUT_DIR] [-o OUTPUT_DIR] [-t TIER] [-j OUTPUT_JSON]
    """

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
                            description="This script takes an directory with ELAN files and "
                                        "slices the audio and output_scripts text in a format ready "
                                        "for our Kaldi pipeline.")
    parser.add_argument("-i", "--input_dir",
                        help="Directory of dirty audio and eaf files",
                        default="working_dir/input/data/")
    parser.add_argument("-o", "--output_dir",
                        help="Output directory",
                        default="../input/output/tmp/")
    parser.add_argument("-t", "--tier",
                        help="Target language tier name",
                        default="Phrase")
    parser.add_argument("-j", "--output_json",
                        help="File path to output json")
    arguments: argparse.Namespace = parser.parse_args()

    # Build output_scripts directory if needed
    if not os.path.exists(arguments.output_dir):
        os.makedirs(arguments.output_dir)

    all_files_in_directory = set(glob.glob(os.path.join(arguments.input_dir, "**"), recursive=True))
    input_eafs_files = find_files_by_extensions(all_files_in_directory, {"*.eaf"})

    annotations_data = []

    for input_eaf_file in input_eafs_files:
        annotations_data.extend(process_eaf(input_eaf_file, arguments.tier))

    write_data_to_json_file(annotations_data, arguments.output_json)


if __name__ == "__main__":
    main()




"""
Test script for validating resample audio script. 

@author Aninda Saha
"""

from kaldi_helpers.input_scripts.resample_audio import *

SCRIPT_PATH = os.path.join(".", "kaldi_helpers", "resample_audio.py")


def test_join_normalised_path() -> None:

    path1: str = os.path.join(".", "cupboard", "narnia")
    path2: str = os.path.join("..", "castle")

    normalised_path: str = join_normalised_path(path1, path2)
    assert normalised_path == os.path.join("cupboard", "castle")


def test_resample_audio() -> None:

    result: subprocess.CompletedProcess = subprocess.run(["python", SCRIPT_PATH, "--corpus", DEFAULT_DATA_DIRECTORY],
                                                         check=True)
    assert result.returncode == 0

    all_files_in_directory: Set[str] = set(glob.glob(os.path.join(DEFAULT_DATA_DIRECTORY, "*.wav"),
                                                     recursive=True))
    print("all files:", all_files_in_directory)

    for file in all_files_in_directory:
        process: subprocess.Popen = subprocess.Popen([SOX_PATH, "--i", "-r", os.path.join(DEFAULT_DATA_DIRECTORY, os.path.basename(file))],
                                stdout=subprocess.PIPE)
        sample_rate: str = process.stdout.read()
        assert int(sample_rate) == 44100





"""
Test script for validating silence audio script. 

@author Aninda Saha
"""

from kaldi_helpers.input_scripts.silence_audio import *

SCRIPT_PATH = os.path.join(".", "kaldi_helpers", "silence_audio.py")

def test_process() -> None:
    assert 1 == 1


def test_silence_audio() -> None:
    assert 1 == 1



from kaldi_helpers.input_scripts.elan_to_json import *

SCRIPT_PATH = os.path.join('.', 'kaldi_helpers', 'elan_to_json.py')


def test_process_eaf():

    # Using script to extract data from test .eaf files
    input_dir = os.path.join("resources", "corpora", "abui_toy_corpus", "data")
    output_dir = os.path.join('.', "test", "testfiles")
    output_json = os.path.join(output_dir, "dirty.json")
    command = ["python", SCRIPT_PATH,
               "--input_dir", input_dir,
               "--output_dir", output_dir,
               "--output_json", output_json]
    result: subprocess.CompletedProcess = subprocess.run(command, check=True)
    assert result.returncode == 0

    with open(output_json) as f:
        contents: List[Dict[str, Union[str, float]]] = json.loads(f.read())
    os.remove(output_json)

    # Manual extraction of transcription data from test .eaf files
    all_files_in_directory: Set[str] = set(glob.glob(os.path.join(input_dir, "*.eaf"),
                                                     recursive=True))
    utterances: List[Dict[str, Union[str, float]]] = []
    for file_name in all_files_in_directory:
        utterances = utterances + process_eaf(file_name, "Phrase")

    assert len(utterances) == len(contents)
    for dictionary in utterances:
        assert dictionary in contents



#!/usr/bin/python3

"""
Extracts transcription information from Praat (*.TextGrid) transcription files and outputs them in json format

Copyright: University of Queensland, 2019
Contributors:
              Scott Heath - (University of Queensland, 2018)
              Nicholas Lambourne - (University of Queensland, 2019)
"""

import argparse
from praatio import tgio
from typing import Dict
from kaldi_helpers.script_utilities import *


def process_textgrid(input_directory: str) -> List[Dict[str, Union[str, int]]]:
    """
    Traverses through the textgrid files in the given directory and extracts 
    transcription information in each tier and creates a list of dictionaries,
    each containing data in the following format: 
                        {'audio_file_name': <file_name>,
                        'transcript': <transcription_label>,
                        'start_ms': <start_time_in_milliseconds>,
                        'stop_ms': <stop_time_in_milliseconds>}
                        
    :param input_directory: directory path containing input_scripts files from where the method
    :return: list of interval data in dictionary form
    """
    intervals: List[Dict[str, Union[str, int]]] = []

    for root, directories, files in os.walk(input_directory):
        for filename in files:
            basename, extension = os.path.splitext(filename)
            if filename.endswith(".TextGrid"):
                text_grid: tgio.Textgrid = tgio.openTextgrid(os.path.join(root, filename))
                speech_tier: tgio.IntervalTier = text_grid.tierDict["Speech"]
                for start, stop, label in speech_tier.entryList:
                    label_word: str = label.replace('"', '')
                    intervals.append({
                        "audio_file_name": os.path.join(".", basename + ".wav"),
                        "transcript": label_word,
                        "start_ms": seconds_to_milliseconds(float(start)),
                        "stop_ms": seconds_to_milliseconds(float(stop))
                    })
    return intervals


def seconds_to_milliseconds(seconds: float) -> int:
    """
    Converts from seconds to milliseconds.
    
    :param seconds: time in seconds
    :return: converted time rounded to nearest millisecond
    """
    return int(seconds * 1000)


def main() -> None:

    """ 
    Run the entire textgrid_to_json.py as a command line utility.
    
    Usage: python3 textgrid_to_json.py [-h] [-i INPUT_DIR] [-o OUTPUT_DIR]
    """

    parser = argparse.ArgumentParser(
        description="Search input_scripts folder for .TextGrid files and convert to JSON on stdout")
    parser.add_argument("-i", "--input_dir", help="The input_scripts data dir", type=str, default="input_scripts/data/")
    parser.add_argument("-o", "--output_dir", help="Output directory", type=str, default="input_scripts/output_scripts/tmp")
    arguments = parser.parse_args()

    if not os.path.exists(arguments.output_dir):
        os.makedirs(arguments.output_dir)

    intervals = process_textgrid(arguments.input_dir)

    result_base_name, name = os.path.split(arguments.output_dir)
    if not name or name == ".":
        outfile_name = "intervals.json"
    else:
        outfile_name = os.path.join(name + ".json")

    output_json = os.path.join(result_base_name, outfile_name)
    write_data_to_json_file(intervals, output_json)


if __name__ == "__main__":
    main()
